package PScm::Closure;

use strict;
use warnings;
use base qw(PScm);

sub new {
    my ($class, $args, $body, $env) = @_;

    bless {
        args => $args,
        body => $body,
        env  => $env,
    }, $class;
}

sub _apply {
    my ($self, $args) = @_;

    my $extended_env =
      $self->{env}->ExtendUnevaluated($self->{args}, $args);
    return $self->{body}->Eval($extended_env);
}

sub as_string {
    my ($self) = @_;
    return PScm::Expr::List->new(
        $self->_symbol,
        $self->{args},
        $self->{body}
    )->as_string;
}

################################
package PScm::Closure::Function;

use base qw(PScm::Closure);

sub Apply {
    my ($self, $form, $env) = @_;

    my $evaluated_args = $form->map_eval($env);
    return $self->_apply($evaluated_args);
}

sub _symbol {
    PScm::Expr::Symbol->new('lambda');
}

#############################
package PScm::Closure::Macro;

use base qw(PScm::Closure);

sub Apply {
    my ($self, $form, $env) = @_;

    my $new_form = $self->_apply($form);
    return $new_form->Eval($env);
}

sub _symbol {
    PScm::Expr::Symbol->new('macro');
}

1;
